#include <avr/io.h>
#include <avr/signal.h>
#include <avr/eeprom.h>
#include <avr/pgmspace.h>


#include "app4adc.h"



#define  ADC_CONTROL_REG            ADCSR
#define  ADC_MUX_REG                ADMUX
#define  NUMBER_OF_ADC_CHANNELS     6
#define  ADC_PORT                   C

#define AD_CLOCK_HZ (ADC_CLOCK_KHZ*1000L)

#if   F_CPU/2 < ((AD_CLOCK_HZ*13)/10)
#define ADC_PRESCALER   1
#elif F_CPU/4 < ((AD_CLOCK_HZ*13)/10)
#define ADC_PRESCALER   2 
#elif F_CPU/8 < ((AD_CLOCK_HZ*13)/10)
#define ADC_PRESCALER   3
#elif F_CPU/16 < ((AD_CLOCK_HZ*13)/10)
#define ADC_PRESCALER   4 
#elif F_CPU/32 < ((AD_CLOCK_HZ*13)/10)
#define ADC_PRESCALER   5 
#elif F_CPU/64 < ((AD_CLOCK_HZ*13)/10)
#define ADC_PRESCALER   6 
#else
#define ADC_PRESCALER   7
#endif



/* REGISTER NAME FORMING */
#ifndef CONCAT1
#define CONCAT1(a, b) CONCAT2(a, b)
#endif

#ifndef CONCAT2
#define CONCAT2(a, b) a ## b
#endif

#define ADC_PORT_OUT_REG       CONCAT1(PORT, ADC_PORT)
#define ADC_PORT_DDR_REG       CONCAT1(DDR, ADC_PORT)
#define ADC_PORT_PIN_REG       CONCAT1(PIN, ADC_PORT)


volatile union adc_union {
                           volatile unsigned int  adc_value; 
                           volatile unsigned char adc_byte[2];
                         } adc;



void  adc_init()
{
  ADC_CONTROL_REG=0;
  ADC_CONTROL_REG|=(1<<ADIF);
#if   ADC_NOISE_REDUCTION == 1 
  ADC_PORT_DDR_REG=0xFF;
  ADC_PORT_OUT_REG=0xFF;
#endif 
  adc_convert(0); // throw out one reading, set NR, etc
  return;
}

/* channel =  0 to 7 (ADC CHANNEL)  ADC uses NO interrupt   */
unsigned int adc_convert(unsigned char channel)
{
     /* Set the appropriate ADMUX bits */    
      ADC_MUX_REG=(ADC_MUX_REG&0xF8)|(channel&7);
#if    ADC_NOISE_REDUCTION == 1 
             ADC_PORT_DDR_REG=~(1<<channel);
             ADC_PORT_OUT_REG=0;
#elif  ADC_NOISE_REDUCTION == 0 
             ADC_PORT_DDR_REG&=~channel;
             ADC_PORT_OUT_REG=~channel;
#endif
             ADC_CONTROL_REG|=ADIF;
             ADC_CONTROL_REG=(1<<ADEN)|(1<<ADSC)|ADC_PRESCALER;
             loop_until_bit_is_set(ADC_CONTROL_REG, ADIF);
             adc.adc_byte[0]=ADCL;
             adc.adc_byte[1]=ADCH;
             return(adc.adc_value);
}



