#include <avr/io.h>
#include "app4.h"
#include "app4uart.h"
#include "app4adc.h"
#include "rtc.h"
#include <stdio.h>

#define AVGCOUNT 4  // temp samples to average
#define DCD ((PINB & 2)!=2)
#define MODEMSTRING "ATV0E0Q1&C1&R1&H0S0=1\r" // answer in one ring

struct temprec
{
  unsigned t;
  unsigned hour;
  unsigned min;
  unsigned sec;
} min,max,temp;


char cmd[66];
int cp; 

void disp(void)
{
    printf("\014%02d:%02d:%02d - %d.%01d\n",
           temp.hour,temp.min,temp.sec,temp.t/10,temp.t%10);
    printf("Min - %02d:%02d:%02d - %d.%01d\n",
           min.hour,min.min,min.sec,min.t/10,min.t%10);
    printf("Max - %02d:%02d:%02d - %d.%01d\n",
           max.hour,max.min,max.sec,max.t/10,max.t%10);
    // print any command in progress for user feedback
    printf("%s\n",cmd);
}


void docmd(void)
{
  cmd[cp]='\0';
  cp=0;
  switch (cmd[0])
    {
    case 'R':
      min.t=0xFFFF;
      max.t=0;
      break;
    case 'D':
      sscanf(cmd+1,"%d",&rtc_day);
      break;
    case 'H':
      sscanf(cmd+1,"%d",&rtc_hour);
      break;
    case 'M':
      sscanf(cmd+1,"%d",&rtc_min);
      break;
    }
  cmd[cp]='\0';  // erase command
}


int main(int argc, char *argv[])
{
  unsigned n;
  UartInit(BAUD_9600);
  rtc_init();
  adc_init();
  // set 2.56V reference
  ADMUX=0xC0;
  UartSetStdio();
  min.t=0xFFFF;
  max.t=0;
  n=rtc_sec;
  // wait a second for the modem to stabalize
  n=rtc_sec;
  while (n==rtc_sec);
  // init modem
  printf(MODEMSTRING);
  // main loop
  while (1)
    {
    unsigned int raw;
    int i;
    // build command string if modem connected
    if (DCD) 
      {
        if (UartReady())
          {
            cmd[cp]=getchar();
            if (cmd[cp]=='\010') cp-=2;  // backspace
            cmd[cp+1]='\0';
            if (cmd[cp]=='\n') 
              docmd();
            else if (cp<sizeof(cmd)-1) cp++;
          }
      }
    else
      cp=0; // kill pending command on hang up
    // wait for one second to elapse
    while (rtc_sec==n) continue; 
    n=rtc_sec;
    // warning: maximum value of adc_convert*AVGCOUNT must fit in unsigned int
    for (i=0,raw=0;i<AVGCOUNT;i++) raw+=adc_convert(0);
    raw/=AVGCOUNT;  // average
    temp.t=(25*raw+5)/10;  // temp is in 1mV units (.1 degree)
    temp.hour=rtc_hour;
    temp.min=rtc_min;
    temp.sec=n;
    // set min and max
    if (min.t>temp.t)
        min=temp;
    if (max.t<temp.t)
        max=temp;

    // If connected, send info
    if (DCD) disp();
    }
}
