#include <avr/io.h>
#include <avr/signal.h>
#include <avr/interrupt.h>

// each interrupt tick is worth 25.6uS and 
// and each raw tick is worth 100nS
// So by shifting the interrupt by a count of 56
// we can make each rtc_tick worth 20uS
// however, using a single clock per raw tick
// makes things hard. So instead, we prescale
// by 8. This makes each raw tick worth 800nS
// and we need 250 of them to make 200uS
unsigned volatile rtc_ticks;  
unsigned volatile rtc_ms;
unsigned volatile rtc_sec;
unsigned volatile rtc_min;
unsigned volatile rtc_hour;
unsigned volatile rtc_day;

// Amount to slide each interrupt
#define TSHIFT ((256-250)+1)


INTERRUPT(SIG_OVERFLOW0)
{
  TCNT0+=TSHIFT;  // shift over so next interrupt is 200uS away
  rtc_ticks++;
  if (rtc_ticks>=5)  // 5*200uS=1mS
    {
      rtc_ms++;
      rtc_ticks-=5;
      if (rtc_ms>=1000) // 1000mS = 1S
        {
          rtc_sec++;
          PORTB^=1;
          rtc_ms-=1000;
          if (rtc_sec>=60)  // 60S = 1 minute
            {
              rtc_min++;
              rtc_sec-=60;
              if (rtc_min>=60) // 60 minutes = 1 hour
                {
                  rtc_hour++;
                  rtc_min-=60;
                  if (rtc_hour>=24) // 24 hours in a day
                    {
                      rtc_day++;
                      rtc_hour-=24;
                    }
                }
            }
        }
    }
}

void rtc_init(void)
{
  DDRB|=1;  // Port B0 is an output (clock out)
  TCCR0=2; // 10MHz clock /8 = 1.25MHz
  TIMSK|=1;
  sei();
}
